

class PRContent:
    
    def __init__(self, title: str ="", body: str ="", content: str = "", file: str =""):
        self.title = title
        self.body = body

        if content:
            self._init_from_content(content)
        elif file:
            with open(file, "r+") as f:
                self._init_from_content(f.read())

    def _init_from_content(self, content: str):
        self.title, _, self.body  = content.strip("\n ").partition("\n")
        self.title = self.title.strip("\n")
        self.body = self.body.strip("\n")
        
    def fill_empty(self, other: "PRContent"):
        if not self.title:
            self.title = other.title
        if not self.body:
            self.body = other.body
        return self

    def __str__(self):
        return self.title + "\n\n" + self.body + "\n"
    
    def __bool__(self):
        return all([self.__dict__[attr] for attr in self.__dict__.keys()])

    def write_to_file(self, fn):
        with open(fn, "w") as f:
            f.write(self.__str__().encode("utf-8"))






import os
import re
import subprocess
import tempfile

from subprocess import TimeoutExpired
from auto_pull_request.utility import quoted_str, stop_timeout_exception
from loguru import logger


SHORT_HASH_LEN = 5
TIMEOUT_SECOND = 30

def _run_shell_command(cmd, input: str =None, raise_on_error: bool=True) -> str:
    assert type(cmd) == list and type(cmd[0]) == str
    new_cmd = " ".join(list(filter((lambda x: x), cmd)))
    
    logger.debug(f"running '{new_cmd}' with input of '{input}'")
    out = ""
    try:
        complete = subprocess.run(
            new_cmd, 
            input=input, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, 
            shell=True, encoding="utf-8", timeout=TIMEOUT_SECOND)
        out = complete.stdout
    except TimeoutExpired:
        logger.debug(f"{cmd} is killed because of TIMEOUTERROR. The output of ")
        raise TimeoutError
    if raise_on_error and complete.returncode:
        logger.error(f"Runned command: {complete.args}. The error output : {out}")
        raise RuntimeError("%s returned %d" % (new_cmd, complete.returncode))
    logger.debug(f"returned code of {new_cmd}: {complete.returncode}; output of that: {out}")
    return out.strip()

    
class Git: 
    
    def __init__(self, protocol:str="https", host:str="github.com"):
        self.username = None
        self.password = None
        self.protocol = protocol
        self.host = host

        self.conf = self.git_conf()
        self.commit_format = {
            "log": "Date:%ci; Author: %an; Commit: %h %n %s%n", 
            "markdown": "## Date:%ci; Author: %an; Commit: %h %n %s%n"
        }
        
    def __str__(self):
        return str(self.__dict__)

    class git_conf:
        
        def get_pr_config(self, pr_subfix, default=None):
            self.get_config("git-pull-request." + pr_subfix, default=default)

        def get_config(self, config_name, default=""):
            if hasattr(self, config_name):
                return getattr(self, config_name)
            try:
                command_list = ["git", "config", "--get", config_name]
                setattr(self, config_name, _run_shell_command(command_list))    
                return getattr(self, config_name)  
            except RuntimeError:
                logger.debug(f"get_config: run command fail: {command_list}.")
                return default

        def set_pr_config(self, pr_subfix, value):
            self.set_config("git-pull-request." + pr_subfix, value)
            
        def set_config(self, config_name, value):
            _run_shell_command(["git", "config", config_name, value])
            
    def get_login_password(self):
        """Get login/password from git credential."""
        if  self.password:
            return self.password

        request = "protocol={}\nhost={}\n".format(self.protocol, self.host)

        out = _run_shell_command(["git", "credential", "fill"], input=request)
        for line in out.split("\n"):
            key, _, value = line.partition("=")
            if key == "username":
                self.username = value.decode()
            elif key == "password":
                self.password = value.decode()
            if self.username and self.password:
                return self.username, self.password

    def approve_login_password(self, user="", password="", host="github.com", protocol="https"):
        """Tell git to approve the credential."""
        request = f"protocol={protocol}\nhost={host}\nusername={user}\npassword={password}\n"
        output = _run_shell_command(["git", "credential", "approve"], input=request)
        logger.info(f"git credential status:{output}")


    def get_matching_remote(self, wanted_url):
        wanted_id = self.get_repository_id_from_url(wanted_url)

        remotes = _run_shell_command(["git", "remote", "-v"])
        for remote in remotes:
            name, remote_url, push_pull = re.split(r"\s", remote)
            if push_pull != "(push)":
                continue
            remote_id = self.fork_repository_id_from_url(remote_url)
            if wanted_id == remote_id:
                return name


    def get_remote_url(self, remote="origin"):
        return self.conf.get_config("remote." + remote + ".url" )

    def get_pr_config_hosttype(self):
        return self.conf.get_pr_config("hosttype")

    def set_pr_config_hosttype(self, hosttype):
        self.conf.set_pr_config("hosttype", hosttype)


    def get_branch_name(self):
        branch = _run_shell_command(["git", "rev-parse", "--abbrev-ref", "HEAD"])
        if branch.upper() == "HEAD":
            raise RuntimeError("Unable to determine current branch")
        return branch

    def get_remote_url_for_branch(self, branch):
        return self.conf.get_config("branch." + branch + ".remote")


    def get_remote_branch_for_branch(self, branch):
        branch = self.conf.get_config("branch." + branch + ".merge")
        if branch.startswith("refs/heads/"):
            return branch[11:]
        return branch

    def get_remote_for_branch(self, branch):
        return self.conf.get_config("branch." + branch + ".remote")

    def get_commit_body(self, commit):
        return _run_shell_command(["git", "show", "--format=%b" , commit, "--"])

    def get_commit_titles(self, begin, end, format):
        return _run_shell_command(
            ["git", "log", "--no-merges", f"--format="+quoted_str(format), f"{begin}..{end}"])

    def get_formated_title(self, head1, head2):
        return  f"Pull request for commit after commit \
            {self.get_object_rsa(head1)[:SHORT_HASH_LEN]} and before {self.get_object_rsa(head2)[:SHORT_HASH_LEN]}"
    
    def get_object_rsa(self, obj):
        return _run_shell_command(["git", "rev-parse", obj])

    def get_formated_logs(self, begin, end):
        return _run_shell_command(
            [
                "git",
                "log",
                "--no-merges",
                "--format=" + quoted_str(self.commit_format["log"]),
                f"{begin}..{end}",
            ])
            
    def run_editor(self, filename:str)-> str:
        editor = _run_shell_command(["git", "var", "GIT_EDITOR"])
        if not editor:
            logger.warning(
                "$EDITOR is unset, you will not be able to edit the pull-request body"
            )
            editor = "cat"
        status = os.system(editor + " " + filename)
        if status != 0:
            raise RuntimeError(f"Editor({editor}) exited with status code {status}" )
        with open(filename, "r") as body:
            return body.read().strip()
      
    def editor_str(self, text: str = ""):
        with tempfile.NamedTemporaryFile() as temp_fp:
            temp_fp.write(text.encode(encoding="utf-8"))
            temp_fp.seek(0)
            _run_shell_command(cmd=["cat",temp_fp.name])
            str = self.run_editor(temp_fp.name)
            return str # explicit naming to keep tempfile alive
        

    def switch_new_branch(self, new_branch, base_branch):
        return _run_shell_command(["git", "checkout", "-b", new_branch, base_branch])

    def switch_branch(self, branch):
        return _run_shell_command(["git", "checkout", branch])
        

    def switch_forcedly_branch(self, branch, base_branch=""):
        try :
            self.switch_branch(branch)
        except RuntimeError:
            if not base_branch:
                base_branch = self.get_branch_name()
            self.switch_new_branch(branch, base_branch)
        else:
            raise RuntimeError(f"Unable create new branch {branch} from branch {base_branch}")
    
    def add_remote_ulr(self, remote, url):
        url = _run_shell_command(["git", "remote", "get-url", remote])
        if not url:
                _run_shell_command(["git", "remote", "add", remote, url])
        logger.info(f"The config has been add. The the url of {remote} remote is {url}.")

    @stop_timeout_exception
    def fetch_branch(self, repo, branch):
        return _run_shell_command(["git", "fetch", repo, branch])
    

    def set_upper_branch(self, remote_branch, local_branch):
        return _run_shell_command(
        ["git", "branch", "-u", remote_branch, local_branch])

    def rebase(self, upstream=None, branch=None):
        if not upstream and branch:
            raise ValueError("If branch isn't empty, fill upstream firstly.") 
        return _run_shell_command(
            ["git", "rebase", upstream, branch]
        )

    @stop_timeout_exception    
    def push(self, remote, source_branch, target_branch, set_upstream=False, ignore_error=False):
        flag = "-u" if set_upstream else ""
        return _run_shell_command(
            ["git", "push", "--tags", flag, remote, f"{source_branch}:{target_branch}"], raise_on_error= not ignore_error)
            
    def clear_status(self) -> bool:
        """check the work tree wether clean
        Please run it at project root.
        """
        try:
            if not _run_shell_command(["git", "diff", "--merge-base", "head", "--", "."])\
            and not _run_shell_command(["git", "diff", "--", "."]):
                return True
        except:
            return False
        return False
    





# -*- encoding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os import fork
import sys
import click
from loguru import logger

from auto_pull_request.pull_request import Auto
from auto_pull_request import __version__

# Creates a GitHub pull-request.
@click.command("pull-request")
@click.option("--debug/--no-debug", default=False, show_default=True,
    help="If true, enable debugging.")
@click.option("--target-url", "-u",
    help="The remote url of branch to send a pull-request to. Default is auto-detected from .git/config. "
    "Target-url should using http or ssh as protocol."
    "There options, target-url, target-remote and target-branch, only needed when you cloned from your repository, or you want "
    "to create a pull-request to another repository.\n" 
    "For example, you can check you target url by \"git config --get \"remote.origin.url\"\""
    )
@click.option("--target-remote", "-r",
    help="The remote name of the target repo int local git, to which we send a pull-request. Default is auto-detected from .git/config. "
    "There options, target-url, target-remote and target-branch, only needed when you cloned from your repository, or you want "
    "to create a pull-request to another repository.\n"
    "As a example, target-remote of a cloned repository from other other people ususally is \"origin\"."
    )
@click.option("--target-branch", "-b",
    help="The remote branch of target branch in local git, to which we send a pull-request. Default value is auto-detected from .git/config. "
    "There options, target-url, target-remote and target-branch, usually needed when you cloned from your repository, or you want "
    "to custom a pull-request.\n"
    )
@click.option("--fork-branch",
    help="The remote branch of fork repo from which we send a pull-request. Default value is auto-detected from .git/config. "
    )
@click.option("--fork-url",
    help="The remote url of fork repo from which we send a pull-request. Default value is upsteam of the current branch. "
 )
@click.option("--fork-remote",
    help="The remote name of fork repo from which we send a pull-request. Default value is upsteam name of the current branch."
 )
@click.option("--title",
    help="Title of the pull request.")
@click.option("--body",
    help="Body of the pull request.")
@click.option(
    "--comment", 
    help="Comment to publish when updating the pull-request")
@click.option(
    "--keep-message/--update-message", default=False, show_default=True,
    help="For a existing pull-request, Don't open an editor to change the pull request body.",
    )
@click.option(
    "--skip-editor/--open-editor", default=False, show_default=True,
    help="If not empty, use parameter of --title and --message instead of " 
    "opening edition for pull-requester content.")
@click.option("--labels", "-l",
    help="The labels to add to the pull request. Can be used multiple times.")
@click.option("--token", prompt=True, type=str,
    help="The personal token of github to log in, which will store in \"git credential\"."
    "If empty, we will promot in terminal to input corresponding infos.\n"
    "How to get you personal token? Please check this https://docs.github.com/en/authentication"
    "/keeping-your-account-and-data-secure/creating-a-personal-access-token")
def main(debug, target_url, target_remote, target_branch, fork_branch, fork_url, fork_remote, title, body, keep_message, labels, comment, skip_editor, token):
    log_info(debug)
    version_lint()
    Auto(
        target_url=target_url,
        target_remote=target_remote,
        target_branch=target_branch,
        fork_branch=fork_branch,
        fork_url=fork_url,
        fork_remote=fork_remote,
        title=title,
        body=body,
        comment=comment,
        keep_message=keep_message,
        labels=labels,
        skip_editor=skip_editor,
        token=token,
        ).run()

def log_info(debug):
    logger.remove()
    level = "DEBUG" if debug else "SUCCESS"
    logger.add(sys.stderr, level=level)
    
def version_lint():
    logger.success(f"Auto-Pull-Request ⭐️{__version__}")


import os
import github 
import glob
import json
from loguru import logger
from urllib import parse

from github.GithubException import GithubException, UnknownObjectException
from github.PullRequest import PullRequest 
from github.Repository import Repository 

from auto_pull_request.content import PRContent
from auto_pull_request.utility import dead_for_resource, dead_for_software, check_true_value_and_logger,format_github_exception
from auto_pull_request.git import Git 


class RepositoryID:
    """Generate host, user and repository name from url"""

    def __init__(self, url: str = ""):
        self.url = url
        parsed = parse.urlparse(url)
        if parsed.scheme == "https": # not empty scheme usually is https
            path = parsed.path.strip()[1:]
            self.host = parsed.netloc
        elif not parsed.scheme and "@" in parsed.path: # we assume that the url is the form of ssh 
            ssh, _, path = parsed.path.partition(':')
            _, _, self.host = ssh.partition("@")
        else:
           raise ValueError("Unsupported scheme {pared.scheme}")
        self.user, self.repo = path.split("/", 1)
        if self.repo.endswith(".git"):
            self.repo = self.repo[:-4]

        if not self.user or not self.repo or not self.host:
            logger.error(f"This url has a empty entity. user: {self.user}, repo:{self.repo}, host:{self.host}")
            dead_for_software() 
            
    def __eq__(self, other:object):
        """
            one excate identical example as follows: 
            >>> parse.urlparse("https://github.com/user/repo.git")
            ParseResult(scheme='https', netloc='github.com', path='/user/repo.git', params='', query='', fragment='')
            >>> parse.urlparse("git@github.com:user/repo.git")
            ParseResult(scheme='', netloc='', path='git@github.com:user/repo.git', params='', query='', fragment='')
            
            host: github.com
            user: volcengine
            repo: volc-sdk-python
        """
        if not isinstance(other, RepositoryID):
            return False
        return (
            self.host.lower() == other.host.lower()
            and self.user.lower() == other.user.lower()
            and self.repo.lower() == other.repo.lower()
        )
    
    def https_url(self):
        return "/".join(["https:/", self.host, self.user, self.repo + ".git"])

    def __str__(self):
        return str(self.__dict__)


class Remote:
    """the object control github repo, corresponding local git remote config

        self.remote_name is local remote name in git.config
        self.remote_branch is the local remote branch syncing with remote repository. Such as, "origin/master".
        self.user_branch: user for head branch of pull-request.
    """
    def __init__(self, remote_name:str="", repo_branch:str="", local_branch:str="", repo:RepositoryID=None, git:Git=None, gh_repo:Repository=None, config=False):
        self._gh_repo = None
        self._repo = None
        self.remote_url = ""
        self.user = ""
        self.config = config
        # TODO0 set the value int git.config
        self.remote_name = remote_name
        self.repo_branch = repo_branch
        self.local_branch = local_branch
        self.git = git
        # set property attr
        self.repo = repo
        self.gh_repo = gh_repo
        
        if config:
            self.set_into_git()
     
    @property
    def gh_repo(self):
        return self._gh_repo

    @gh_repo.setter
    def gh_repo(self, new_gh_repo:Repository):
        if not new_gh_repo or self._gh_repo:
            return

        self._gh_repo = new_gh_repo
        self.remote_url  = self.remote_url or self._gh_repo.clone_url
        
        self.repo = RepositoryID(self._gh_repo.clone_url)
        

    @property
    def repo(self):
        return self._repo

    @repo.setter
    def repo(self, new_repo:RepositoryID):
        if not new_repo or self.repo:
            assert self.repo == new_repo, "Errors, origin: {self.repo}  != repo_from_gh: {repo} "
            return 
        self._repo = new_repo
        self.user = self._repo.user
        self.remote_url = self.remote_url or new_repo.https_url()  #TODO use https or ssh url.
        self.remote_name =  self.remote_name or new_repo.user

    def addRemote(self, other:"Remote"):
        for attr in other.__dict__:
            self.__dict__[attr] =  self.__dict__[attr] or other.__dict__[attr]

    @property
    def remote_branch(self):
        return "/".join([self.remote_name, self.repo_branch])
    
    @property
    def user_branch(self):
        return ":".join([self.user, self.repo_branch])

    def check_integrity(self, name="Remote"):
        try:
            skip_list = ["config"]
            for attr in self.__dict__ :
                if not self.__dict__[attr] and attr not in skip_list:
                    raise ValueError(f"During checking the integrity of {name}, "
                    f"found the {attr} is empty. May you provide relation variables in command line.")
        except ValueError as e:
            logger.error(e)
            dead_for_software()

    #todo set move ?
    def set_into_git(self):
        self.git.add_remote_ulr(self.repo.user, self.remote_url)
        
    #todo  create instance from git config?
    @classmethod
    def create_from_git(self):
        pass

    def exist_repo_branches(self, branch):
        self.branches = self.gh_repo.get_branches()
        return branch in self.branches

    def clear_local(self):
        if not self.git.clear_status():
            logger.error("Please commit local changes firstly")
            dead_for_resource()

    def pull(self):
        self.clear_local()
        if not self.exist_repo_branches(self.repo_branch):
            return

        self.git.fetch_branch(self.remote_name, self.local_branch)
        try:
            self.git.rebase(self.remote_branch, self.local_branch)
        except RuntimeError:
            logger.error(
                f"During the rebasing {self.local_branch} from {self.remote_branch}, "
                "it is likely that your change has a merge conflict. "
                "You may resolve it by `git add . ;git rebase --continue` command. "
                "Once done run `git pull-request' again. "
                "If you want to abort conflict resolution, run `git rebase --abort'."
            )
            dead_for_resource()

    def push(self, ignore_error=False):
        self.clear_local()
        self.git.push(self.remote_name, self.local_branch, self.repo_branch, ignore_error=ignore_error)
    
    def __str__(self):
        return " ".join([item + ": \'" + str(self.__dict__[item]) + "\'"for item in self.__dict__ if item != "git"])


class Auto:
    """ 
        Main Vars:
        self.target_url: most important url to assign remote target repository. If don't provided, we will auto choose local remote from current branch.
        self.fork_remote: the Remote of fork from target repo. And self.fork_remote.repo also set to the git remote fork name.
        self.target_branch: the branch name of self.target_remote corresponding self.local_branch. AKA, the remote branch. such as "master"
        self.local_remote: local remote stores local info in git.config.
    """
    def __init__(self,
        target_url="",
        target_remote="",
        target_branch="",
        fork_branch="",
        fork_url="",
        fork_remote="",
        title="",
        body="",
        keep_message=None,
        comment="",
        labels=None,
        skip_editor="",
        token=""):
        self.git = Git()
        self.content = PRContent(title, body)
        self.keep_message = keep_message
        self.comment = comment
        self.labels = labels
        self.skip_editor = skip_editor
        self.token = token

        self.local_remote_is_fork = True if target_url else False # The rightness is dependent on the user.
        # accpet option parameters and basic info
        self.target_remote = Remote(
            git = self.git,
            repo = RepositoryID(target_url) if target_url else None, 
            remote_name = target_remote, 
            repo_branch = target_branch,
        )
        self.fork_remote = Remote(
            git = self.git,
            repo_branch = fork_branch,
            repo = RepositoryID(fork_url) if fork_url else None,
            remote_name = fork_remote,
        )
        # accept local parameters
        branch = self.git.get_branch_name()
        self.local_remote = self.get_local_remote(branch)
        if self.local_remote_is_fork:
            self.fork_remote.addRemote(self.local_remote)
            self.target_remote.local_branch = branch
        else:
            self.target_remote.addRemote(self.local_remote)
            self.fork_remote.local_branch = branch
        # accept github paramters
        self._init_github()
        self._init_credential()
        logger.info(f"target_remote info: {self.target_remote}")
        logger.info(f"fork_remote info: {self.fork_remote}")
        # check
        self.target_remote.check_integrity("target_remote")
        self.fork_remote.check_integrity("fork_remote")

        if self.fork_remote.repo == self.target_remote.repo:
            logger.error(f"Detect the remote target repo is the forked repository, which is {self.fork_remote.remote_url}. Please assigned --target-remote, --target-url with options.")
            dead_for_resource()
        logger.success("The Initialization completed-_^")

    def get_local_remote(self, branch):
        remote_name = self.git.get_remote_for_branch(branch)
        remote_url = self.git.get_remote_url(remote_name)
        return Remote(
            repo =  RepositoryID(remote_url) if remote_url else None ,
            remote_name = remote_name,
            repo_branch = self.git.get_remote_branch_for_branch(branch),
            local_branch = branch,
        )
        
    def _init_credential(self):
        if not self.token:
            self.username, self.token = self.git.get_login_password()
            check_true_value_and_logger(self.token, f"Unable to find your token of {self.fork_remote.repo.host}. "
                "Make sure you have a git credential working.", os.EX_UNAVAILABLE)
        else:
            self.git.approve_login_password(host=self.fork_remote.repo.host, user=self.fork_remote.repo.user, password=self.token) #TODO hide debug info of the token
        logger.info(f"Found user: {self.fork_remote.repo.user} password: <redacted> in host {self.fork_remote.repo.host}")

    def _init_github(self):
        assert self.target_remote.repo, "target repo must not empty"
        
        try:
            self.gh = github.Github(self.token)
        except UnknownObjectException as e:
            logger.error(format_github_exception("login with the token", e))
            dead_for_resource()
        try:
            self.gh_user = self.gh.get_user()
        except UnknownObjectException as e:
            logger.error(format_github_exception(f"get githut login user-{self.fork_remote.repo.user}", e))
            dead_for_resource()
        
        try:
            self.target_remote.gh_repo = self.gh.get_user(self.target_remote.repo.user).get_repo(self.target_remote.repo.repo) 
        except UnknownObjectException as e:
            logger.error(format_github_exception(f"get github target repo with user \
                {self.target_remote.repo.user}/{self.target_remote.repo.repo} ", e))
            dead_for_resource()
        try:
            self.fork()
        except UnknownObjectException as e:
            logger.info(format_github_exception(f"fork repository from {self.target_remote.repo.user}/{self.target_remote.repo.repo}", e))
            dead_for_resource()

    def _get_pull_request_template(self):
        filename = "PULL_REQUEST_TEMPLATE*"
        pr_template_paths = [
            filename,
            ".github/PULL_REQUEST_TEMPLATE/*.md",
            ".github/PULL_REQUEST_TEMPLATE/*.txt",
            os.path.join(".github", filename),
            os.path.join("docs", filename),
            filename.lower(),
            ".github/pull_request_template/*.md",
            ".github/pull_request_template/*.txt",
            os.path.join(".github", filename.lower()),
            os.path.join("docs", filename.lower()),
        ]
        for path in pr_template_paths:
            templates = glob.glob(path)
            for template_path in templates:
                if os.path.isfile(template_path):
                    with open(template_path) as t:
                        return t.read()

    def fork(self):
        """run fork forcedly every time"""
        try:
            self.fork_remote.gh_repo = self.target_remote.gh_repo.create_fork()
        except GithubException as e:
            if  e.status == 403\
                and "forking is disabled" in e.data["message"]:
                logger.error("Forking is disabled on target repository.")
                dead_for_resource()
        
        assert self.fork_remote.repo and self.fork_remote.gh_repo, "fork repo should not empty"

        logger.success(f"Forked repository: {self.fork_remote.gh_repo.clone_url}", )
        
    def run(self):
        self.sync()
        self.push_pr()
        logger.info("Done~ ^_^")

    def sync(self):
        self.target_remote.pull()
        self.fork_remote.pull()
        self.fork_remote.push(ignore_error=False)
        logger.success("Syncing remote and local repositories successes😄")
    
    def push_pr(self):
        self.fill_content()
        pulls = list(self.target_remote.gh_repo.get_pulls(base=self.target_remote.repo_branch, head=self.fork_remote.user_branch))
        if not pulls:
            pr = self.create_pr()
        else:
            pr = pulls[0]
            if len(pulls) > 1:
                logger.info(f"Pull-request({pulls[1:]}) has been ignored.")
            self.upgrade_pr_info(pr)

    def create_pr(self):
        try:
            pr = self.target_remote.gh_repo.create_pull(
                base=self.target_remote.repo_branch, head=self.fork_remote.user_branch, title=self.content.title, body=self.content.body
            )
        except github.GithubException as e:
            logger.error(format_github_exception(
                f"create pull request with base={self.target_remote.repo_branch} "
                f"and head={self.fork_remote.user_branch}", e))
            dead_for_resource()
        logger.success(f"Pull-request created: {pr.html_url}")
        return pr

    def fill_content(self):
        """If self.content has empty value, Get title and body summary for patches between 2 commits.
        """
        if self.content:
            return

        title = self.git.get_formated_title(self.target_remote.repo_branch, self.target_remote.local_branch)
        body = self.git.get_formated_logs(self.target_remote.repo_branch, self.target_remote.local_branch)
        if not self.skip_editor:
            edited = self.git.editor_str(str(PRContent(title, body)))
            self.content.fill_empty(PRContent(content=edited))
        else:
            self.content.fill_empty(PRContent(title, body))

    def upgrade_pr_info(self, pr:PullRequest):
        self.content.fill_empty(PRContent(pr.title, pr.body))
       
        if not self.keep_message:
            self.fill_content()
            pr.edit(title=self.content.title, body=self.content.body)
            logger.debug("Updated pull-request title and body")
            
        if self.comment:
            self.target_remote.gh_repo.get_issue(pr.number).create_comment(self.comment)
            logger.debug(f'Pull-request {pr.number} Commented: "{self.comment}"', )
        if self.labels:
            pr.add_to_labels(*self.labels)
            logger.debug(f"Pull-request {pr.number} added labels \"{self.labels}\"", )

        logger.success(f"Success to update pr in {pr.html_url}")



import operator
import os
import json
from loguru import logger

from github import GithubException

def split_and_remove_empty_lines(s):
    return filter(operator.truth, s.split("\n"))

def check_true_value_and_logger(value, msg:str="", exit_code=None, *error_values):
    check_and_logger(value, msg, exit_code, *[None, zero_value(value)])

def check_and_logger(value, msg:str="", exit_code=None, *error_values):
    for checked in error_values:
        if checked == value:
            logger.critical(msg + "value:{value}")
            if exit_code: 
                dead_for_resource()
            else:
                return


def format_github_exception(action:str , e: GithubException):
    url = e.data.get("documentation_url", "GitHub documentation")
    errors_msg = "; ".join(
        json.dumps(error) for error in e.data.get("errors", {}) # type: ignore
    )
    return f"Unable to {action}: {e.data.get('message')} ({e.status}). Errors: {errors_msg}."\
            f"Check {url} for more information."

def zero_value(value):
    if value != None:
        return type(value)()
    return None

def dead_for_resource():
    exit(os.EX_UNAVAILABLE)

def dead_for_software():
    exit(os.EX_SOFTWARE)

def quoted_str(text:str):
    return "\"" + text + "\""

def stop_timeout_exception(func):
    def try_solve_fun(*args, **kwargs):
        try:
            func(*args, **kwargs)
        except TimeoutError as e:
            logger.error("If command that solves https run too long, maybe it's a platform problem, such as Darwin. "
            "You can re-do to try again. Probably success.")
            dead_for_resource()
    
    return try_solve_fun


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['auto_pull_request']

package_data = \
{'': ['*']}

install_requires = \
['PyGithub>=1.55,<2.0',
 'attrs>=21.2.0,<22.0.0',
 'click>=8.0.1,<9.0.0',
 'loguru>=0.5.3,<0.6.0',
 'poetry-dynamic-versioning==0.13.1']

entry_points = \
{'console_scripts': ['apr = auto_pull_request.parser:main']}

setup_kwargs = {
    'name': 'auto-pull-request',
    'version': '7.0.16',
    'description': '',
    'long_description': None,
    'author': 'ruth-seven',
    'author_email': '1098057570@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
